#include <sys/types.h>
#include <sys/socket.h>

#include "tcpxm.h"
#include "mcp.h"

/*
 * send TCP control data
*/
send_control(int s, int cmd, void *arg) {
  int r;
  char buf[10];
  char str[64];

  sprintf(buf, "%d", cmd);
  if(write(s, buf, sizeof(buf)) < 0) {
    printf("send_control: write failed\n");
    exit(1);
  }

  switch(cmd) {
    case MCP_SRCPORT:
      sprintf(buf, "%d", (int)arg);
      r = write(s, buf, sizeof(buf));
      break;
    case MCP_DSTPORT:
      sprintf(buf, "%d", (int)arg);
      r = write(s, buf, sizeof(buf));
      break;
    case MCP_FILESIZE:
      sprintf(buf, "%lu", (unsigned long)arg);
      r = write(s, buf, sizeof(buf));
      break;
    case MCP_FILENAME:
      bzero(str, sizeof(str));
      memcpy(str, (char *)arg, strlen((char *)arg));
      r = write(s, str, sizeof(str));
      break;
    case MCP_STATUS:
      sprintf(buf, "%d", (int)arg);
      r = write(s, buf, sizeof(buf));
      break;
    case MCP_MSG:
      bzero(str, sizeof(str));
      memcpy(str, (char *)arg, strlen((char *)arg));
      r = write(s, str, sizeof(str));
      break;
    case MCP_PROTOCOL:
      sprintf(buf, "%d", (int)arg);
      r = write(s, buf, sizeof(buf));
      break;
  }

  if(r < 0) {
    printf("send_control: write failed\n");
    exit(1);
  }
}

/*
 * receive TCP control data
*/
int recv_control(int s, struct mcpcntl_s *mcpcntl) {

  char buf[10];
  int cmd, r;

  if(read(s, buf, sizeof(buf)) < 0) {
    printf("recv_control: read failed\n");
    exit(1);
  }
  cmd = atoi(buf);

  switch(cmd) {
    case MCP_SRCPORT:
      r = read(s, buf, sizeof(buf));
      mcpcntl->srcport = atoi(buf);
      break;
    case MCP_DSTPORT:
      r = read(s, buf, sizeof(buf));
      mcpcntl->dstport = atoi(buf);
      break;
    case MCP_FILESIZE:
      r = read(s, buf, sizeof(buf));
      mcpcntl->bytecount = strtoul(buf, NULL, 10);
      break;
    case MCP_FILENAME:
      r = read(s, mcpcntl->filename, sizeof(mcpcntl->filename));
      break;
    case MCP_STATUS:
      r = read(s, buf, sizeof(buf));
      mcpcntl->status = atoi(buf);
      break;
    case MCP_MSG:
      r = read(s, mcpcntl->msg, sizeof(mcpcntl->msg));
      break;
    case MCP_PROTOCOL:
      r = read(s, buf, sizeof(buf));
      mcpcntl->protocol = atoi(buf);
      break;
  }

  if(r < 0) {
    printf("recv_control: read failed\n");
    exit(1);
  }
}

/*
 * find first free udp/tcp port starting at 'sport'
 *   sport = suggested port
 *   *s = socket that will hold port
 * returns chosen port
*/
int getport(int mcpproto, int sport, int *s) {
  int r=-1, type, protocol;
  static int count=0;
  static struct mysockaddr_in saddr;

  if(mcpproto == MCP_TCPXM) {
    type = SOCK_DGRAM;
    protocol = IPPROTO_UDP;
  } else {
    type = SOCK_STREAM;
    protocol = IPPROTO_IP;
  }

  /* creating test socket */
  close(*s);
  if((*s = socket(PF_INET, type, protocol)) == -1) {
    perror("error creating test socket");
  }
  bzero((char *)&saddr, sizeof(saddr));
  saddr.sin_family = PF_INET;
  saddr.sin_addr.s_addr = htonl(INADDR_ANY);

  /* keep binding until we find free port */
  sport--;
  do {
    bzero((char *)&saddr, sizeof(saddr));
    saddr.sin_family = PF_INET;
    saddr.sin_addr.s_addr = htonl(INADDR_ANY);

    sport = ++sport > 65535 ? 1025 : sport;
    saddr.sin_port = htons(sport);
    r = bind(*s, (struct sockaddr *)&saddr, sizeof(saddr));
  } while(r < 0);

  return(sport);
}
