//#define CCODEDATE "karl"
/*
 * mcp.c
 *
 * Multicast Copy Client
 * Karl Jeacle
 * University of Cambridge
 *
 * karl.jeacle@cl.cam.ac.uk
 */

#include <stdio.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/time.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/signal.h>

#include "tcpxm.h"
#include "mcp.h"

#ifndef TCPXM
netconn_connectxm(){}
netconn_waitclose(){}
netconn_mbytes(){return(0);};
netconn_ubytes(){return(0);};
#endif

struct netconn *mconn, **uconn;

int
main(int argc, char **argv)
{
  struct mysockaddr_in *tcpaddr, *tcpaddr2;
  struct mcpcntl_s mcpcntl;
  struct hostent *hp;
  struct ip_addr *remoteaddr, group;
  struct netbuf *buf;
  int j, r, *s, *s2, len, numhosts;
  unsigned long i, filelen, filelen2;
  unsigned long maxtransfer=(2^32)-65535;
  int fd, txmode;
  int uflag=0, Uflag=0, fflag=0, sflag=0, tflag=0, nflag=0, 
      qflag=0, vflag=0, Vflag=0, Tflag=0, mflag=0;
  char ch, filedata[BLOCKSIZE], filename[64];
  struct stat sb;
  int mcp_srcport=MCPSRCPORT, mcp_dstport=MCPDSTPORT, highport=0;
  int suggestdstport, checksock;
  char **tcphost, *colon;
  int *tcpport, portstried=0;
  unsigned long bytes=0, ubytesper=0, mbytesper=0;
  unsigned long ubytes=0, mbytes=0, tmbytes=0, ubytes_early=0;
  unsigned long numbytes=1000000;
  unsigned long bytesrcvd=0, ubytesrcvd=0, mbytesrcvd=0;
  unsigned long totubytesrcvd=0, totmbytesrcvd=0;
  struct timeval start, end;
  struct timezone tz;
  float secs, usecs;
  void handler();

  memset(&group, 0, sizeof(group));

  while((ch = getopt(argc, argv, "g:hmn:qst:TuUvV")) != -1) {
    switch (ch) {
      case 'g':
        if((hp = (struct hostent *)gethostbyname(optarg)) == NULL) {
          printf("error resolving %s\n", optarg);
          exit(1);
        }
        memcpy(&group.addr, hp->h_addr, hp->h_length);
        break;
      case 'h':
        helpusage();
        break;
      case 'm':
        mflag = 1;
        break;
      case 'n':
        numbytes = strtoul(optarg, NULL, 10);
        nflag = 1;
        break;
      case 'q':
        qflag = 1;
        break;
      case 's':
        sflag = 1;
        break;
      case 't':
        tflag = atoi(optarg);
        break;
      case 'T':
        Tflag = 1;
        break;
      case 'u':
        uflag = 1;
        break;
      case 'U':
        Uflag = 1;
        break;
      case 'v':
        vflag = 1;
        break;
      case 'V':
        Vflag = 1;
        break;
      case '?':
      default:
        usage();
    }
  }
  argc -= optind;
  argv += optind;

  if(mflag+uflag+Uflag+Tflag > 1) {
    printf("Must choose between one of -m, -u, -U and -T!\n");
    exit(1);
  }

  if(Vflag && (Uflag || Tflag)) {
    printf("-V not available with -U or -T!\n");
    exit(1);
  }

  if(argc < 2) {
    usage();
  }

  /* catch ctrl-c to cleanup */
/*
  if(signal(SIGINT, handler) == SIG_ERR) {
    perror("error initializing signal handler");
    exit(1);
  }
*/

  /* open data file */
  strcpy(filename, nflag ? "/dev/zero" : argv[0]);
  if((fd = open(filename, O_RDONLY)) < 0) {
    printf("couldn't open file %s\n", filename);
    exit(1);
  }

  /* check if file too large! */
  if(nflag) {
    filelen = numbytes;
  } else {
    fstat(fd, &sb);
    filelen = sb.st_size;
  }

  if(filelen <= 0 || filelen >= maxtransfer) {
    printf("Sorry, transfer exceeds max of %lu bytes!\n", maxtransfer);
    exit(1);
  } else {
    filelen2 = (((filelen / BLOCKSIZE) + 1 ) * BLOCKSIZE);
  }

  if(vflag)
    printf("sending file %s = %lu bytes\n", filename, filelen);

  /* process destination address arguments */
  numhosts = argc - 1;
  remoteaddr = malloc(numhosts * sizeof(struct ip_addr));
  uconn = malloc(numhosts * sizeof(struct netconn *));
  tcpaddr = malloc(numhosts * sizeof(struct mysockaddr_in));
  s = malloc(numhosts * sizeof(int));
  tcphost = malloc(numhosts * sizeof(char *));
  tcpport = malloc(numhosts * sizeof(int));

  if(Tflag) {
    tcpaddr2 = malloc(numhosts * sizeof(struct mysockaddr_in));
    s2 = malloc(numhosts * sizeof(int));
  }

  /* parse host:port args */
  for(i=0; i<numhosts; i++) {
    tcphost[i] = malloc(strlen(argv[i+1])+1);
    if(strchr(argv[i+1], ':')) {
      strcpy(tcphost[i], strtok(argv[i+1], ":"));
      tcpport[i] = atoi(strtok(NULL, ":"));
    } else {
      strcpy(tcphost[i], argv[i+1]);
      tcpport[i] = MCPDTCPPORT;
    }
  }


  /* resolve hostnames and work out TCP & TCP-XM IP addrs */
  for(i=0; i<numhosts; i++) {
    if((hp = (struct hostent *)gethostbyname(tcphost[i])) == NULL) {
      printf("error resolving %s\n", tcphost[i]);
      exit(1);
    }

    /* TCP-XM remote addrs */
    memcpy(&remoteaddr[i].addr, hp->h_addr, hp->h_length);

    /* TCP remote addrs for control connections */
    bzero((char *) &tcpaddr[i], sizeof(struct mysockaddr_in));
    memcpy(&tcpaddr[i].sin_addr, hp->h_addr, hp->h_length);
    tcpaddr[i].sin_family      = AF_INET;
    tcpaddr[i].sin_port        = htons(tcpport[i]);

    if(Tflag) {
      /* TCP remote addrs for MultiTCP connections*/
      bzero((char *) &tcpaddr2[i], sizeof(struct mysockaddr_in));
      memcpy(&tcpaddr2[i].sin_addr, hp->h_addr, hp->h_length);
      tcpaddr2[i].sin_family      = AF_INET;
    }
  }

  if(!tflag) {
    if(vflag)
      printf("tcp control connecting...\n");

    /* open TCP control connections */
    for(i=0; i<numhosts; i++) {

      if((s[i] = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
        perror("create control socket failed");
        exit(1);
      }

      if(connect(s[i], (struct sockaddr *)&tcpaddr[i], sizeof(struct mysockaddr_in)) < 0) {
        printf("control connect to %s failed: %s\n",tcphost[i],strerror(errno));
        exit(1);
      }
    }
    if(vflag)
      printf("tcp control connected\n");

    /* get free UDP data port for tcp-xm (suggest MCPSRCPORT) */
    mcp_srcport = getport(Tflag ? MCP_TCP : MCP_TCPXM, MCPSRCPORT, &checksock);

    /* send control data to destinations */
    for(i=0; i<numhosts; i++) {
      send_control(s[i], MCP_SRCPORT, mcp_srcport);
      send_control(s[i], MCP_DSTPORT, mcp_dstport);
      send_control(s[i], MCP_FILESIZE, filelen);
      send_control(s[i], MCP_FILENAME, basename(filename));
      send_control(s[i], MCP_PROTOCOL, Tflag ? MCP_TCP : MCP_TCPXM);
    }

    /* loop negotiating common UDP dstport for tcpxm with destinations */
    do {
      suggestdstport = mcp_dstport;

      /* get back results of dstport availability */
      for(i=0; i<numhosts; i++) {
        recv_control(s[i], &mcpcntl);
        if(vflag)
          printf("dest %d offering port %d\n", i+1, mcpcntl.dstport);
        highport = mcpcntl.dstport > highport ? mcpcntl.dstport : highport;
      }

      /* check if our suggestion agrees with all servers */
      if(suggestdstport != highport) {

        if(portstried++ > 10) {
          /* tell servers we're giving up */
          for(i=0; i<numhosts; i++) {
            send_control(s[i], MCP_STATUS, MCP_FAIL);
          }
          printf("Couldn't negotiate common dest port (tried 10+ values)\n");
          exit(1);
        }

        if(vflag)
          printf("servers didn't like %d, trying %d\n",suggestdstport,highport);

        mcp_dstport = highport;

        /* try suggesting highport to servers */
        for(i=0; i<numhosts; i++) {
          send_control(s[i], MCP_DSTPORT, mcp_dstport);
        }
      }

    } while(suggestdstport != highport);


    if(vflag)
      printf("servers agree on dstport %d\n", mcp_dstport);

    /* send dstport negotiation ok to servers */
    for(i=0; i<numhosts; i++) {
      send_control(s[i], MCP_STATUS, MCP_OK);
    }

    /* receive ready-to-go from servers */
    for(i=0; i<numhosts; i++) {
      recv_control(s[i], &mcpcntl); /* status */
      if(mcpcntl.status != MCP_OK) {
        recv_control(s[i], &mcpcntl); /* msg */
        printf("%s: %s\n", tcphost[i], mcpcntl.msg);
        exit(1);
      }
    }

    if(vflag)
      printf("tcpxm data ports: cli=%d srv=%d\n", mcp_srcport, mcp_dstport);

    close(checksock);
  }

  /* clear screen for multi-line variable display */
  if(Vflag) {
    printf("%c[2J", 0x1B); /* clear screen */
    printf("%c[;H", 0x1B); /* home */
  }

  if(Tflag) {

    if(vflag)
      printf("MultiTCP data connecting...\n");

    for(i=0; i<numhosts; i++) {

      if((s2[i] = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
        perror("create data socket failed");
        exit(1);
      }

      tcpaddr2[i].sin_port = htons(mcp_dstport);

      if(connect(s2[i], (struct sockaddr *)&tcpaddr2[i], sizeof(struct mysockaddr_in)) < 0) {
        printf("tcp data connect port %u to %s failed: %s\n", mcp_dstport, tcphost[i], strerror(errno));
        exit(1);
      }
    }

  } else {
    /* init lwip stack */
    tcpxm_init(tflag, mcp_srcport, mcp_dstport);
 
    /* create netconn */
    if(Uflag) {
      for(i=0; i<numhosts; i++) {
        uconn[i] = netconn_new(NETCONN_TCP);
      }
    } else {
#ifdef TCPXM
      mconn = netconn_new(NETCONN_TCPXM);
#else
      printf("Can't multicast with rlwip use -u flag!\n");
      exit(1);
#endif
    }

    /* tcpxm connect */
    if(Uflag) {
      if(vflag)
        printf("tcp data connecting...\n");
      for(i=0; i<numhosts; i++) {
        if((r = netconn_connect(uconn[i], &remoteaddr[i], 7)) < 0) {
          printf("connect failed: %s\n", lwip_strerr(r));
          exit(1);
        }
      }
    } else {
      if(vflag)
        printf("tcp-xm data connecting...\n");

      if(uflag) {
        txmode = 1; /* tx_unicast_group_only */
      } else {
        txmode = 0; /* tx_unicast_group */
      }
      if(mflag)
        txmode = 2; /* tx_multicast_only */

      if((r=netconn_connectxm(mconn,remoteaddr,numhosts, &group, 7, txmode))<0){
        printf("connectxm failed: %s\n", lwip_strerr(r));
        exit(1);
      }
    }
  } /* if(Tflag) */

  if(vflag)
      printf("data connected\n");

  /* allow time for destination IGMP join latency */
/*
  if(!uflag && !uflag) {
    sleep(1);
  }
*/

  /* start timer */
  gettimeofday(&start, &tz);

  /* write data to network */
  for(i=0; i < filelen2; i+=BLOCKSIZE) {
    len = (i+BLOCKSIZE) > filelen ? filelen-i : BLOCKSIZE;
    bytes += len;

    if((r = read(fd, filedata, len)) < 0) {
      perror("file read failed");
      exit(1);
    }

    if(Uflag || Tflag) {
      for(j=0;j<numhosts;j++) {
        if(Uflag) {
          if((r = netconn_write(uconn[j], filedata, len, NETCONN_COPY)) < 0) {
            printf("netconn_write failed: %s\n", lwip_strerr(r));
            exit(1);
          }
        } else { /* Tflag */
          if(write(s2[j], filedata, len) < 0) {
            printf("tcp data write failed\n");
            exit(1);
          }
        }
      }
/*
      if(vflag)
        printf("writing %d bytes from %d to %d\n", len, i+1, i+len);
*/

    } else {

      if((r = netconn_write(mconn, filedata, len, NETCONN_COPY)) < 0) {
        printf("netconn_write failed: %s\n", lwip_strerr(r));
        exit(1);
      }
    } /* if(u/T) */

      /* print multi-line variable display or progress meter */
      if(Vflag && !(Uflag || Tflag)) {
#ifdef TCPXM
        printvariables(numhosts, tcphost, mconn);
#endif
      }
      if(!Vflag && !qflag && !sflag) {
        printstatus(bytes, filelen, start);
      }


  } /* for() */

  /* newline after status bar */
  if(vflag)
    printf("\n");

  /* send TCP close... */
  if(vflag)
    printf("data closing...\n");
  if(Uflag || Tflag) {
    for(i=0;i<numhosts;i++) {
      if(Uflag) {
        netconn_close(uconn[i]);
      } else {
        close(s2[i]);
      }
    }
  } else {
    netconn_close(mconn);
  }

  /* allow time for ubytes/mbytes counts to be complete! */
  /* do this by waiting for TIME_WAIT state */
  if(Uflag) {
    for(i=0;i<numhosts;i++) {
      netconn_waitclose(uconn[i], 10, 2000);
      if(uconn[i]==NULL) {
        printf("conn closed but null!\n");
        exit(1);
      }
    }
  } else if(!Tflag) {
    ubytes_early = netconn_ubytes(mconn);
    netconn_waitclose(mconn, 10, 10000);
    if(mconn==NULL) {
      printf("conn closed but null!\n");
      exit(1);
    }
  }

  /* stop timer */
  gettimeofday(&end, &tz);
  usecs = abs(end.tv_usec - start.tv_usec);
  secs = (end.tv_sec - start.tv_sec) + usecs/1000000 ;

  if(vflag)
    printf("data closed\n");

  /* delete connection and record bytes sent*/
  if(Uflag) {
    for(i=0;i<numhosts;i++) {
      //mbytes += netconn_mbytes(uconn[i]);
      ubytes += netconn_ubytes(uconn[i]);
      netconn_delete(uconn[i]);
    }
  } else if(!Tflag) {
    mbytes = netconn_mbytes(mconn);
    ubytes = netconn_ubytes(mconn);
    netconn_delete(mconn);
  }

  /* use earlier bytecount in case pcb gone after waitclose */
  if(ubytes == 0) {
    ubytes = ubytes_early;
  }

  close(fd);

  if(!tflag) {
    /* get bytes received counts from destinations */
    for(i=0; i<numhosts; i++) {
      recv_control(s[i], &mcpcntl); bytesrcvd  = mcpcntl.bytecount;
      recv_control(s[i], &mcpcntl); ubytesrcvd = mcpcntl.bytecount;
      recv_control(s[i], &mcpcntl); mbytesrcvd = mcpcntl.bytecount;

      if(bytesrcvd != filelen) {
        printf("%s received %lu bytes not %lu\n", 
          tcphost[i], bytesrcvd, filelen);
      } else {
          if(vflag)
            printf("%s received %lu bytes (u=%u m=%u)\n",
              tcphost[i], bytesrcvd, ubytesrcvd, mbytesrcvd);
      }

      totubytesrcvd += ubytesrcvd;
      totmbytesrcvd += mbytesrcvd;
    }

    /* close TCP control connections */
    if(vflag)
      printf("tcp control closing...\n");
    for(i=0; i<numhosts; i++) {
      close(s[i]);
    }
    if(vflag)
      printf("tcp control closed\n");
  }


  /* percentage u/m traffic */
  if(ubytes+mbytes > 0) {
    /* scale up mbytes for percentage comparison */
    mbytes *= numhosts;
    /* can't count parallel uni/mcast as real mcast */
    if(mbytes > ubytes) {
      tmbytes = mbytes - ubytes;
    } else {
      tmbytes = 0;
    }
    ubytesper = 100*((double)ubytes  / (ubytes + tmbytes));
    mbytesper = 100*((double)tmbytes / (ubytes + tmbytes));
  }

  /* print results */
  if(vflag) {
    printf("wrote=%d tx'd=%d (%d uni + %d multi) u=%d%% m=%d%%\nsecs=%.2f bytes/s=%.2f kbits/s=%.2f\n\n",
      bytes, ubytes+mbytes, ubytes, mbytes, ubytesper, mbytesper,
      secs, bytes/secs, (bytes*8/1000)/secs);
  } else {
    if(sflag) {
      if(tflag)
        printf("%.2f %d %d\n", (bytes*8/1000)/secs, ubytes, mbytes);
      else
        printf("%.2f %d %d\n", (bytes*8/1000)/secs,totubytesrcvd,totmbytesrcvd);
    }
  }

  /* kbits/sec summary after status bar */
  if(!vflag && !Vflag && !qflag && !sflag) {
    printf("%c[%dC", 0x1B, 60); /* move across cols */
    printf("%.2f kbits/s\n", (bytes*8/1000)/secs);
  }

#ifndef TCPXM
  /* allow time for tcp stack to finish up */
  sleep(1);
#endif

  return(0);
}


void handler()
{
  int status;

  wait(&status);  /* Wait for SIGINT */
  printf("ctrl-c aborting\n");
  netconn_close(mconn);
  netconn_waitclose(mconn, 10, 10000);
  netconn_delete(mconn);
  exit(1);
}

usage() {
  printf("usage: mcp [-hmqsTuUvV] [-g group] [-n bytes] [-t tap#] <file> <host[:port]>...\n");
  exit(1);
}

helpusage() {
  printf("usage: mcp [-hqsuUvV] [-g group] [-n bytes] [-t tap#] <file> <host[:port]>...\n");
  printf("Code version: %s\n", CODEDATE);
  printf("-h         == this help\n");
  printf("-g <group> == multicast group address\n");
  printf("-m         == force multicast (no unicast fallback)\n");
  printf("-n <bytes> == num bytes to send from /dev/zero (<file> ignored)\n");
  printf("-q         == quiet - no progress bar\n");
  printf("-s         == print short bytes/sec summary\n");
  printf("-T         == multiple TCP streams (not TCP-XM)\n");
  printf("-t <devno> == /dev/tap# instead of udp\n");
  printf("-u         == TCP-XM group unicast\n");
  printf("-U         == multiple lwIP TCP streams\n");
  printf("-v         == verbose\n");
  printf("-V         == multi-line variable display\n");
  exit(0);
}

#ifdef TCPXM
/*
 * multi-line variable display
*/
printvariables(int numhosts, char **hostnames, struct netconn *mconn) {
  struct tcp_pcb *pcb;
  char ch, host[11];
  int i=0;
  static int notfirst=0;

  if(notfirst++) {
    printf("%c[%dA", 0x1B, numhosts); /* move up rows */
  } else {
    printf("host       tx ubytes     mbytes     cwnd   swnd   sa sv rtime rto rttest mseg%%\n");
    printf("----       -- ------     ------     ----   ----   -- -- ----- --- ------ -----\n");
  }

  for(pcb=mconn->pcb.tcp; pcb!=NULL; pcb=pcb->nextm) {

    ch = (pcb->txmode == 1) ? 'm' : 'u';

    strncpy(host, hostnames[i++], 10);

    printf("%-10s %-2c %-10u %-10u %-6u %-6u %-2u %-2u %-5u %-3u %-6u %-3u\n",
      host,
      ch,
      pcb->ubytessent,
      pcb->mbytessent,
      pcb->cwnd,
      pcb->snd_wnd,
      pcb->sa,
      pcb->sv,
      pcb->rtime,
      pcb->rto,
      pcb->rttest,
      pcb->msegsntper
    );
  }
}
#endif

/*
 * status bar display
*/
printstatus(unsigned long bytes, unsigned long filelen, struct timeval start) {
  int i, percent, asterisks, min, sec;
  char s[41], time[6];
  struct timezone tz;
  struct timeval now;
  float secs;

  gettimeofday(&now, &tz);
  secs = now.tv_sec - start.tv_sec;

  /* convert from seconds elapsed to estimate of seconds left */
  secs *= (double)(filelen-bytes) / bytes;

  min = secs / 60;
  sec = (int)secs % 60;

  if(secs > 5999) {
    strcpy(time, "00:00");
  } else {
    sprintf(time, "%2d:%2d", min, sec);
    if(min < 10) time[0] = '0';
    if(sec < 10) time[3] = '0';
  }

  memset(&s, 0, sizeof(s));
  percent = 100 * ((double)bytes / filelen);

  for(i=0; i<(4*percent/10); i++) {
    s[i] = '*';
  }

  printf("%3d%% |%-40s| %-11u ETA %s\n", percent, s, bytes, time);
  printf("%c[%dA", 0x1B, 1); /* move up row */
}
