#ifndef __LWIPOPTS_H__
#define __LWIPOPTS_H__

/*
#if defined(sun)
#define BYTE_ORDER BIG_ENDIAN
#endif
*/

#define IGMP_MULTICAST		1

/* ---------- Memory options ---------- */
/* MEM_ALIGNMENT: should be set to the alignment of the CPU for which
   lwIP is compiled. 4 byte alignment -> define MEM_ALIGNMENT to 4, 2
   byte alignment -> define MEM_ALIGNMENT to 2. */
// kj solaris needs 4
#define MEM_ALIGNMENT           4

/* MEM_SIZE: the size of the heap memory. If the application will send
a lot of data that needs to be copied, this should be set high. */
// mem_malloc failed at 1M with 14 receivers
#define MEM_SIZE                10000000

/* MEMP_NUM_PBUF: the number of memp struct pbufs. If the application
   sends a lot of data out of ROM (or other static memory), this
   should be set high. */
#define MEMP_NUM_PBUF           32
/* MEMP_NUM_UDP_PCB: the number of UDP protocol control blocks. One
   per active UDP "connection". */
#define MEMP_NUM_UDP_PCB        8
/* MEMP_NUM_TCP_PCB: the number of simulatenously active TCP
   connections. */
#define MEMP_NUM_TCP_PCB        20
/* MEMP_NUM_TCP_PCB_LISTEN: the number of listening TCP
   connections. */
#define MEMP_NUM_TCP_PCB_LISTEN 16
/* MEMP_NUM_TCP_SEG: the number of simultaneously queued TCP
   segments. */
#define MEMP_NUM_TCP_SEG        2000
/* MEMP_NUM_SYS_TIMEOUT: the number of simulateously active
   timeouts. */
//#define MEMP_NUM_SYS_TIMEOUT    3
#define MEMP_NUM_SYS_TIMEOUT    60

/* The following four are used only with the sequential API and can be
   set to 0 if the application only will use the raw API. */
/* MEMP_NUM_NETBUF: the number of struct netbufs. */
#define MEMP_NUM_NETBUF         8
/* MEMP_NUM_NETCONN: the number of struct netconns. */
#define MEMP_NUM_NETCONN        20
/* MEMP_NUM_APIMSG: the number of struct api_msg, used for
   communication between the TCP/IP stack and the sequential
   programs. */
#define MEMP_NUM_API_MSG        2048
/* MEMP_NUM_TCPIPMSG: the number of struct tcpip_msg, which is used
   for sequential API communication and incoming packets. Used in
   src/api/tcpip.c. */
// out of pool 9 with 2048
#define MEMP_NUM_TCPIP_MSG      4096

/* These two control is reclaimer functions should be compiled
   in. Should always be turned on (1). */
#define MEM_RECLAIM             1
#define MEMP_RECLAIM            1

/* ---------- Pbuf options ---------- */
/* PBUF_POOL_SIZE: the number of buffers in the pbuf pool. */
// udpif pbuf malloc fail on 2000
#define PBUF_POOL_SIZE          4000

/* PBUF_POOL_BUFSIZE: the size of each pbuf in the pbuf pool. */
#define PBUF_POOL_BUFSIZE       1500

/* PBUF_LINK_HLEN: the number of bytes that should be allocated for a
   link level header. */
#define PBUF_LINK_HLEN          16

/** SYS_LIGHTWEIGHT_PROT
 * define SYS_LIGHTWEIGHT_PROT in lwipopts.h if you want inter-task protection
 * for certain critical regions during buffer allocation, deallocation and memory
 * allocation and deallocation.
 */
#define SYS_LIGHTWEIGHT_PROT           1

/* ---------- TCP options ---------- */
#define LWIP_TCP                1
#define TCP_TTL                 255

/* Controls if TCP should queue segments that arrive out of
   order. Define to 0 if your device is low on memory. */
#define TCP_QUEUE_OOSEQ         1

/* TCP Maximum segment size. */
#define TCP_MSS                 1432
// should be 1500-20-8-20-20=1432

/* TCP sender buffer space (bytes). multiples of 1432 */
#define TCP_SND_BUF             64440
//#define TCP_SND_BUF             31504
//#define TCP_SND_BUF             15752
//#define TCP_SND_BUF             7160

/* TCP sender buffer space (pbufs). This must be at least = 2 *
   TCP_SND_BUF/TCP_MSS for things to work. */
//kj note: tcp_snd_buf is what actually matters, tcp_write and
//tcp_enqueue check it first before worrying about queue len
#define TCP_SND_QUEUELEN        2 * TCP_SND_BUF/TCP_MSS

/* TCP receive window. */
#define TCP_WND                 64440

/* Maximum number of retransmissions of data segments. */
#define TCP_MAXRTX              12

/* Maximum number of retransmissions of SYN segments. */
#define TCP_SYNMAXRTX           4

/* TCP writable space (bytes). This must be less than or equal
   to TCP_SND_BUF. It is the amount of space which must be
   available in the tcp snd_buf for select to return writable */
#define TCP_SNDLOWAT            TCP_SND_BUF/2


/* The TCP timer interval in milliseconds. */
/* local = 10ms OK          */
/* LAN = 50ms hang, 75ms OK */
#define TCP_TMR_INTERVAL        75


/* ---------- ARP options ---------- */
#define ARP_TABLE_SIZE 10
#define ARP_QUEUEING 1
/**
 * - If enabled, cache entries are generated for every kind of ARP traffic or
 * broadcast IP traffic. This enhances behaviour for sending to a dynamic set
 * of hosts, for example if acting as a gateway.
 * - If disabled, cache entries are generated only for IP destination addresses
 * in use by lwIP or applications. This enhances performance if sending to a small,
 * reasonably static number of hosts. Typically for embedded devices.
 */
//decprecated #define ETHARP_ALWAYS_INSERT 0


/* ---------- IP options ---------- */
/* Define IP_FORWARD to 1 if you wish to have the ability to forward
   IP packets across network interfaces. If you are going to run lwIP
   on a device with only one network interface, define this to 0. */
#define IP_FORWARD              0

/* If defined to 1, IP options are allowed (but not parsed). If
   defined to 0, all packets with IP options are dropped. */
// ifndef avoids netinet/in.h redefine
#ifndef IP_OPTIONS
#define IP_OPTIONS              1
#endif

/* IP reassembly and segmentation.These are orthogonal even
 * if they both deal with IP fragments */
#define IP_REASSEMBLY     1
#define IP_FRAG           1

/* ---------- ICMP options ---------- */
#define ICMP_TTL                255

#define LWIP_RAW	0

/* ---------- DHCP options ---------- */
/* Define LWIP_DHCP to 1 if you want DHCP configuration of
   interfaces. DHCP is not implemented in lwIP 0.5.1, however, so
   turning this on does currently not work. */
#define LWIP_DHCP               0

/* 1 if you want to do an ARP check on the offered address
   (recommended). */
#define DHCP_DOES_ARP_CHECK     1

/* ---------- UDP options ---------- */
#define LWIP_UDP                1
#define UDP_TTL                 255






#ifndef DEMO_DEBUG
#define DEMO_DEBUG      DBG_OFF
#endif

#ifndef UDP_DEBUG
#define UDP_DEBUG        DBG_OFF
#endif

#ifndef SLIP_DEBUG 
#define SLIP_DEBUG       DBG_OFF
#endif

#ifndef DHCP_DEBUG 
#define DHCP_DEBUG       DBG_OFF
#endif

#ifndef TAPIF_DEBUG
#define TAPIF_DEBUG      DBG_OFF
#endif

#ifndef ETHARP_DEBUG
#define ETHARP_DEBUG     DBG_OFF
#endif

#ifndef SOCKETS_DEBUG
#define SOCKETS_DEBUG    DBG_OFF
#endif

#ifndef ICMP_DEBUG
#define ICMP_DEBUG       DBG_OFF
#endif

#ifndef IP_REASS_DEBUG
#define IP_REASS_DEBUG   DBG_OFF
#endif

#ifndef IGMP_DEBUG
#define IGMP_DEBUG      DBG_OFF
#endif

#ifndef NETIF_DEBUG
#define NETIF_DEBUG      DBG_OFF
#endif

#ifndef PBUF_DEBUG
#define PBUF_DEBUG       DBG_OFF
#endif

#ifndef API_LIB_DEBUG
#define API_LIB_DEBUG    DBG_OFF
#endif

#ifndef API_MSG_DEBUG
#define API_MSG_DEBUG    DBG_OFF
#endif

#ifndef INET_DEBUG
#define INET_DEBUG       DBG_OFF
#endif

#ifndef IP_DEBUG
#define IP_DEBUG         DBG_OFF
#endif

#ifndef MEM_DEBUG
#define MEM_DEBUG        DBG_OFF
#endif

#ifndef MEMP_DEBUG
#define MEMP_DEBUG       DBG_OFF
#endif

#ifndef SYS_DEBUG
#define SYS_DEBUG        DBG_OFF
#endif

#ifndef TCP_DEBUG
#define TCP_DEBUG        DBG_OFF
#endif

#ifndef TCP_FR_DEBUG
#define TCP_FR_DEBUG     DBG_OFF
#endif

#ifndef TCP_RTO_DEBUG
#define TCP_RTO_DEBUG    DBG_OFF
#endif

#ifndef TCP_REXMIT_DEBUG
#define TCP_REXMIT_DEBUG DBG_OFF
#endif

#ifndef TCP_RST_DEBUG
#define TCP_RST_DEBUG    DBG_OFF
#endif

#ifndef TCPIP_DEBUG
#define TCPIP_DEBUG      DBG_OFF
#endif

#ifndef UDPIF_DEBUG
#define UDPIF_DEBUG      DBG_OFF
#endif

#ifndef TCP_QLEN_DEBUG
#define TCP_QLEN_DEBUG   DBG_OFF
#endif

#ifndef TCP_WND_DEBUG
#define TCP_WND_DEBUG    DBG_OFF
#endif

#ifndef TCP_CWND_DEBUG
#define TCP_CWND_DEBUG   DBG_OFF
#endif

#ifndef TCP_OUTPUT_DEBUG
#define TCP_OUTPUT_DEBUG DBG_OFF
#endif

#ifndef TCP_INPUT_DEBUG
#define TCP_INPUT_DEBUG  DBG_OFF
#endif

#ifndef TCPXM_DEBUG
#define TCPXM_DEBUG      DBG_OFF
#endif

#ifndef DBG_TYPES_ON
#define DBG_TYPES_ON 	DBG_OFF
#endif

#endif /* __LWIPOPTS_H__ */
