/*-----------------------------------------------------------------------------------*/
/* tcpxm.c
 *
 * The initialisation functions for a library
 *
 * You may need to configure this file to your own needs - it is only an example
 * of how lwIP can be used as a self initialising shared library.
 *
 * In particular, you should change the gateway, ipaddr, and netmask to be the values
 * you would like the stack to use.
 */
/*-----------------------------------------------------------------------------------*/
#define __LWIP_INET_H__

#include <unistd.h>

#include "lwip/sys.h"
#include "lwip/mem.h"
#include "lwip/memp.h"
#include "lwip/pbuf.h"
#include "lwip/tcp.h"
#include "lwip/tcpip.h"
#include "lwip/netif.h"
#include "lwip/stats.h"

#include "netif/tapif.h"
#include "netif/etharp.h"
#include "udpif.h"

#include "tcpxm.h"


static void
tcpip_init_done(void *arg)
{
  sys_sem_t *sem;
  sem = arg;
  sys_sem_signal(*sem);
}

void tcpxm_init(int octet, int sport, int dport) {

  struct ip_addr ipaddr, netmask, gateway;
  struct netif *netif;
  sys_sem_t sem;
  char hostname[128];
  struct hostent *hp;
  err_t (* initfunc)(struct netif *netif);
#if defined(__FreeBSD__)
  struct tapif *tapif;
#endif

  stats_init();
  sys_init();
  mem_init();
  memp_init();
  pbuf_init();
  
  sem = sys_sem_new(0);
  tcpip_init(tcpip_init_done, &sem);
  sys_sem_wait(sem);
  sys_sem_free(sem);
  
  netif_init();
#ifdef TCPXM
  igmp_init();
#endif

  if(octet != 0) {

    /* use tapif */
    IP4_ADDR(&ipaddr,  192,168,octet,octet);
    IP4_ADDR(&gateway, 192,168,octet,250);
    IP4_ADDR(&netmask, 255,255,255,0);

#if defined(__FreeBSD__)
    initfunc = tapif_init;
#else
    printf("tap compiled for FreeBSD only!\n");
    exit(1);
#endif

  } else {
    /* use udpif */
    /* set lwip ip address to be host's ip address */
    gethostname(hostname, sizeof(hostname));
    hp = (struct hostent *)gethostbyname(hostname);
    memcpy(&ipaddr, hp->h_addr, sizeof(ipaddr));

    /* put udpif source port into netmask */
    memcpy(&netmask, &sport, sizeof(sport));

    /* put udpif dest port into gateway */
    memcpy(&gateway, &dport, sizeof(dport));

    initfunc = udpif_init;
  }
  
  netif = mem_malloc(sizeof(struct netif));
  netif_add(netif, &ipaddr, &netmask, &gateway, NULL, initfunc, tcpip_input);
  if(netif == NULL) {
    printf("tcpxm: netif_add failed\n");
    exit(1);
  }

#if defined(__FreeBSD__)
  if(octet != 0) {
    tapif = netif->state;
    tapif->ethaddr->addr[1] = octet;
    tapif->ethaddr->addr[2] = octet;
    tapif->ethaddr->addr[3] = octet;
    tapif->ethaddr->addr[4] = octet;
    tapif->ethaddr->addr[5] = octet;
  }
#endif

  netif_set_default(netif);
  netif_set_up(netif);
}
